""" Problem 4 -- Poker Hands """

# Constants for hand rankings
ROYAL_FLUSH = 9
STRAIGHT_FLUSH = 8
FOUR_OF_KIND = 7
FULL_HOUSE = 6
FLUSH = 5
STRAIGHT = 4
THREE_OF_KIND = 3
TWO_PAIRS = 2
ONE_PAIR = 1
HIGH_CARD = 0

def main():
    """ Opens file and loops through all pairs of hands."""
    player1wins = 0
    player2wins = 0
    inFile = open('prob4.in', 'r')
    numberOfHands = int(inFile.readline())

    for count in xrange(numberOfHands):
        hand1, hand2 = readHands(inFile)
        #print hand1, hand2
        winner = determineWinner(hand1, hand2)
        if winner == 1:
            player1wins += 1
        elif winner == 2:
            player2wins += 1
        else:
            print "TIE"
    print "Player %d won %d hands.\n" % (1, player1wins)
    print "Player %d won %d hands.\n" % (2, player2wins)

def checkForStraight(cardCounts):
    """ Checks for straight by looking for a
        slice of 5 consecutive cards"""
    for start in xrange(2, 11):
        slice = cardCounts[start:start+5]
        if slice == [1, 1, 1, 1, 1]:
            return True
    return False    

def evaluate(hand):
    """ Determines the rank of a hand. """
    cardCounts = []
    suitCounts = [0, 0, 0, 0]
    # count number of each kind of card and # of each suit
    for i in xrange(15):
        cardCounts.append(0)
    for card in hand:
        cardCounts[card[0]] += 1
        if card[1] == 'S':
            suitCounts[0] += 1
        elif card[1] == 'C':
            suitCounts[1] += 1
        elif card[1] == 'D':
            suitCounts[2] += 1
        elif card[1] == 'H':
            suitCounts[3] += 1

    # Build sorted list of tuples: (count, kind)
    # e.g., (3, 9) means 3 nines
    countOfKinds = []
    for card in xrange(2, 15):
        countOfKinds.append((cardCounts[card],card))
    countOfKinds.sort(reverse=True)
    
    isFlush = max(suitCounts) == 5
    isStraight = checkForStraight(cardCounts)

    # Determine rank of the hand
    if isFlush and isStraight:
        if countOfKinds[0][1] == 14:
            handRank = ROYAL_FLUSH
        else:
            handRank = STRAIGHT_FLUSH
    elif countOfKinds[0][0] == 4:
        handRank = FOUR_OF_KIND
    elif countOfKinds[0][0] == 3 and countOfKinds[1][0] == 2:
        handRank = FULL_HOUSE
    elif isFlush:
        handRank = FLUSH
    elif isStraight:
        handRank = STRAIGHT
    elif countOfKinds[0][0] == 3:
        handRank = THREE_OF_KIND
    elif countOfKinds[0][0] == 2 and countOfKinds[1][0] == 2:
        handRank = TWO_PAIRS
    elif countOfKinds[0][0] == 2:
        handRank = ONE_PAIR
    else:
        handRank = HIGH_CARD
    return handRank, countOfKinds

def determineWinner(hand1, hand2):
    hand1Rank, countOfKindsHand1 = evaluate(hand1)
    hand2Rank, countOfKindsHand2 = evaluate(hand2)
    if hand1Rank < hand2Rank:
        return 2
    elif hand1Rank > hand2Rank:
        return 1
    else: #tied Rank
        if countOfKindsHand1 < countOfKindsHand2:
            return 2
        elif countOfKindsHand1 > countOfKindsHand2:
            return 1
        else: # tied Hands
            return 0

def readHands(inFile):
    cards = inFile.readline().strip().split(' ')
    hand1 = []
    hand2 = []
    count = 0
    for card in cards:
        count += 1
        if card[0] == 'A':
            cardValue = 14
        elif card[0] == 'K':
            cardValue = 13
        elif card[0] == 'Q':
            cardValue = 12
        elif card[0] == 'J':
            cardValue = 11
        elif card[0] == 'T':
            cardValue = 10
        else:
            cardValue = int(card[0])
        if count <= 5:
            hand1.append((cardValue, card[1]))
        else:
            hand2.append((cardValue, card[1]))
            
    return hand1, hand2
            
main()
